import typing
import datetime
import pydantic

import schemaless.rid
import schemaless.bases
import schemaless.config
import schemaless.http_client


class DeltaModel(pydantic.BaseModel):
    operation: str
    path: str
    old_value: pydantic.Json
    new_value: pydantic.Json
    hash: str

    @classmethod
    def from_api_response(cls, response: dict) -> "DeltaModel":
        return cls(
            **response,
        )


class CommitModel(pydantic.BaseModel):
    deltas: typing.List[DeltaModel]
    branch: str
    parents: typing.Union[typing.List[schemaless.rid.RID], None] = None
    domain_rid: str
    rid: schemaless.rid.RID
    hash: str
    created_at: int
    modified_at: int
    deleted_at: typing.Union[int, None] = None

    model_config = pydantic.ConfigDict(use_enum_values=True)

    def get_created_at_datetime(self) -> datetime.datetime:
        return datetime.datetime.fromtimestamp(self.created_at)

    def get_updated_at_datetime(self) -> datetime.datetime:
        return datetime.datetime.fromtimestamp(self.modified_at)

    def get_deleted_at_datetime(self):
        return datetime.datetime.fromtimestamp(self.deleted_at)

    @classmethod
    def from_api_response(cls, response: dict) -> "CommitModel":
        return cls(
            **response,
        )


class Domain(schemaless.bases.Handler):
    def __init__(self, http_client: schemaless.bases.BaseHTTP):
        self.http_client = http_client

    @classmethod
    def from_config(cls, config: schemaless.config.Config) -> "Domain":
        return cls(
            http_client=schemaless.http_client.HTTP.from_config(config),
        )



import typing

import pydantic

import schemaless.bases
import schemaless.config
import schemaless.http_client
import schemaless.delta
import schemaless.rid


class PathHistoryItem(pydantic.BaseModel):
    created_at: int
    delta: schemaless.delta.DeltaModel


class PathHistoryResponse(pydantic.BaseModel):
    history: typing.List[PathHistoryItem]


class SubmitObjectBody(pydantic.BaseModel):
    domain_rid: schemaless.rid.RID
    rid: schemaless.rid.RID
    object: dict[str, typing.Any]


class DeltaListResponse(pydantic.BaseModel):
    delta: typing.List[schemaless.delta.DeltaModel]


class VersionControl(schemaless.bases.Handler):
    def __init__(self, http_client: schemaless.bases.BaseHTTP):
        self.http_client = http_client

    @classmethod
    def from_config(cls, config: schemaless.config.Config) -> "VersionControl":
        return cls(
            http_client=schemaless.http_client.HTTP.from_config(config),
        )

    async def submit_object(
        self,
        domain_rid: schemaless.rid.RID,
        rid: schemaless.rid.RID,
        obj: dict[str, typing.Any],
    ) -> DeltaListResponse:
        response = await self.http_client.post(
            url="/api/v1/vc/object/submit",
            json=SubmitObjectBody(
                domain_rid=domain_rid,
                rid=rid,
                object=obj,
            ).model_dump(),
        )

        return response.parse_to_model(DeltaListResponse)

    async def get_snapshot(
        self,
        domain_rid: schemaless.rid.RID,
        rid: schemaless.rid.RID,
        model: typing.Optional[typing.Type[schemaless.bases.T]] = None,
    ) -> typing.Union[typing.Dict[str, typing.Any], typing.Type[schemaless.bases.T]]:
        response = await self.http_client.get(
            url="/api/v1/vc/object/snapshot",
            params={
                "domain_rid": domain_rid.to_string(),
                "rid": rid.to_string(),
            },
        )

        if model is None:
            return response.data()

        return response.parse_to_model(model)

    async def get_history_of_path(
        self,
        domain_rid: schemaless.rid.RID,
        rid: schemaless.rid.RID,
        path: str,
    ) -> PathHistoryResponse:
        response = await self.http_client.get(
            url="/api/v1/vc/history/path",
            params={
                "domain_rid": domain_rid.to_string(),
                "rid": rid.to_string(),
                "path": path,
            },
        )
        print(response.data())
        return response.parse_to_model(PathHistoryResponse)



import abc
import typing

import schemaless.config


class Handler(abc.ABC):
    @classmethod
    @abc.abstractmethod
    def from_config(cls, config: schemaless.config.Config) -> "Handler":
        ...


T = typing.TypeVar("T")


class BaseHTTPResponse(abc.ABC):
    @abc.abstractmethod
    def parse_to_model(self, model: typing.Type[T]) -> T:
        ...

    @abc.abstractmethod
    def data(self) -> typing.Dict[str, typing.Any]:
        ...


class BaseHTTP(abc.ABC):
    @classmethod
    @abc.abstractmethod
    def from_config(cls, config: schemaless.config.Config) -> "BaseHTTP":
        ...

    @abc.abstractmethod
    async def _create_response(self, method: str, **kwargs) -> BaseHTTPResponse:
        ...

    @abc.abstractmethod
    async def get(self, url: str, params: dict = None) -> BaseHTTPResponse:
        ...

    @abc.abstractmethod
    async def post(self, url: str, json: dict = None) -> BaseHTTPResponse:
        ...

    @abc.abstractmethod
    async def put(self, url: str, json: dict = None) -> BaseHTTPResponse:
        ...

    @abc.abstractmethod
    async def delete(self, url: str) -> BaseHTTPResponse:
        ...



import enum
import typing

import httpx

import pydantic
import schemaless.bases
import schemaless.config

API_KEY_KEY = "X-API-Key"


class HTTPResponse(schemaless.bases.BaseHTTPResponse):
    def __init__(self, status_code: int, headers=None, json=None):
        self.status_code = status_code
        self.headers = headers
        self.json = json

    @classmethod
    def from_httpx_response(cls, response: httpx.Response) -> "HTTPResponse":
        return cls(
            status_code=response.status_code,
            headers=response.headers,
            json=response.json(),
        )

    def parse_to_schemaless_model(self) -> "SchemalessHTTPResponseModel":
        if not self.json:
            raise ValueError("Response does not contain json")

        return SchemalessHTTPResponseModel(**self.json)

    def parse_to_model(
        self, model: typing.Type[schemaless.bases.T]
    ) -> schemaless.bases.T:
        schemaless_response = self.parse_to_schemaless_model()
        if schemaless_response.status == Status.Success:
            if isinstance(schemaless_response.data, list):
                return model.validate_python(schemaless_response.data)
            return model(**schemaless_response.data)
        elif schemaless_response.status == Status.Error:
            raise SchemalessException(schemaless_response.data)

        raise ValueError("Response does not contain json")

    def data(self) -> typing.Dict[str, typing.Any]:
        return self.parse_to_schemaless_model().data


class Status(str, enum.Enum):
    Success = "Success"
    Error = "Error"
    Fail = "Fail"


class SchemalessHTTPResponseModel(pydantic.BaseModel):
    status: Status
    data: typing.Any


class SchemalessException(Exception):
    pass


class HTTP(schemaless.bases.BaseHTTP):
    def __init__(self, base_url: str, api_key: str):
        super().__init__()
        self.api_key = api_key
        self.http_client = httpx.AsyncClient(base_url=base_url)

    @classmethod
    def from_config(cls, config: schemaless.config.Config) -> "HTTP":
        return cls(
            base_url=config.get_base_url(),
            api_key=config.get_api_key(),
        )

    async def _create_response(self, method: str, **kwargs) -> HTTPResponse:
        response = await getattr(self.http_client, method)(**kwargs)
        return HTTPResponse.from_httpx_response(response)

    async def get(self, url: str, params: dict = None) -> HTTPResponse:
        return await self._create_response(
            "get",
            url=url,
            params=params,
            headers={API_KEY_KEY: self.api_key},
        )

    async def post(self, url: str, json: dict = None) -> HTTPResponse:
        return await self._create_response(
            "post",
            url=url,
            json=json,
            headers={API_KEY_KEY: self.api_key},
        )

    async def put(self, url: str, json: dict = None) -> HTTPResponse:
        return await self._create_response(
            "put",
            url=url,
            json=json,
            headers={API_KEY_KEY: self.api_key},
        )

    async def delete(self, url: str) -> HTTPResponse:
        return await self._create_response(
            "delete",
            url=url,
            headers={API_KEY_KEY: self.api_key},
        )
